package ass.excercises.cvi8.formbuilder.swing;

import java.awt.Component;
import java.awt.GridLayout;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.function.Supplier;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ButtonModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import org.apache.commons.lang3.Validate;

import ass.excercises.cvi8.formbuilder.IFormBuilder;

public class SwingFormBuilder implements IFormBuilder {

	private final Map<String, Supplier<String>> nameMap = new HashMap<>();
	
	private final JFrame frame = new JFrame();
	private final JPanel mainPanel = new JPanel();
	private final JButton btnSave = new JButton("Save");
	
	public SwingFormBuilder() {
		frame.add(mainPanel);
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}
	
	@Override
	public IFormBuilder addTextField(String name, String label) {
		final JTextField txtField = new JTextField();
		addField(name, label, txtField, () -> txtField.getText());
		
		// notice fluent interface pattern
		return this;
	}

	@Override
	public IFormBuilder addCheckbox(String name) {
		final JCheckBox checkBox = new JCheckBox(name);
		addField(name, name, checkBox, () -> checkBox.isSelected()+"");
		return this;
	}

	@Override
	public IFormBuilder addRadio(String name, String[] values) {
		final List<JRadioButton> radios = new LinkedList<>();
		final ButtonGroup btnGroup = new ButtonGroup();
		for(int i = 0; i < values.length; i++) {
			JRadioButton radio = new JRadioButton();
			radio.setActionCommand(values[i]);
			radios.add(radio);
			btnGroup.add(radio);
			
			addLabeledComponent(values[i], radio);
		}
		nameMap.put(
				name, 
				() -> 
					Optional.ofNullable(btnGroup.getSelection())
					.map(ButtonModel::getActionCommand)
					.orElse("N/A")
		);
		return this;
	}

	public void show() {
		// notice delegation over inheritance
		frame.setVisible(true);
		frame.pack();
	}

	protected void addField(String name, String label, Component c, Supplier<String> sup) {
		Validate.isTrue(!nameMap.containsKey(name), "Name '"+name+"' already present!");
		addLabeledComponent(label, c);
		nameMap.put(name, sup);
	}
	
	protected void addLabeledComponent(String label, Component c) {
		JLabel lbl = new JLabel(label);
		JPanel panel = new JPanel();
		panel.setLayout(new GridLayout(2, 1));
		panel.add(lbl);
		panel.add(c);
		mainPanel.add(panel);
	}
	
	@Override
	public SwingForm build() {
		mainPanel.add(btnSave);
		return new SwingForm(frame, btnSave, nameMap);
	}

}
