package ass.excercises.cvi8.formbuilder.task;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang3.exception.ExceptionUtils;

import com.google.gson.Gson;

import ass.excercises.cvi8.formbuilder.ExampleUtil;
import ass.excercises.cvi8.formbuilder.IForm;
import ass.excercises.cvi8.formbuilder.html.HtmlFormBuilder;
import fi.iki.elonen.NanoHTTPD;
import fi.iki.elonen.NanoHTTPD.Response.Status;

public class HttpServerAppExample extends NanoHTTPD {

	public static final int PORT = 8080;
	
	public HttpServerAppExample() throws IOException {
		super(PORT);
		start(NanoHTTPD.SOCKET_READ_TIMEOUT, false);
		System.out.println("\nRunning! Point your browsers to http://localhost:"+PORT+"/ \n");
	}

	public static void main(String[] args) {
		try {
			new HttpServerAppExample();
		} catch (IOException ioe) {
			System.err.println("Couldn't start server:\n" + ioe);
		}
	}

	@Override
	public Response serve(IHTTPSession session) {
		// prepare and build HTML form from entity using reflection
		ReflectionFormBuilder<Entity> reflectionBuilder = new ReflectionFormBuilder<>(Entity.class);
		HtmlFormBuilder formBuilder = new HtmlFormBuilder();
		IForm form  = reflectionBuilder.build(formBuilder);
		String formHTML = form.render().orElse("<h1>Could not render HTML form!</h1>");
		
		if("GET".equals(session.getMethod().name())) {
			return newFixedLengthResponse(formHTML);
		}else if("POST".equals(session.getMethod().name())) {
			Map<String, String> params = new HashMap<>();
			try {
				session.parseBody(params);
				String postData = session.getQueryParameterString();
				
				Entity entity = reflectionBuilder.instanceFromPOST(postData);
				
				Gson gson = new Gson();
				
				// convert entity to json using reflection
				String json = gson.toJson(entity);
				
				return newFixedLengthResponse(Status.OK, "application/json", json);
			} catch (IOException | ResponseException e) {
				String stackTrace = ExceptionUtils.getStackTrace(e);
				return newFixedLengthResponse(Status.INTERNAL_ERROR, "text/html", stackTrace);
			}
		}else {
			return newFixedLengthResponse(Status.BAD_REQUEST, "text/html", "");
		}
	}

}
